//=============================================================================
// SurvivalCoopLogicHandler.
//=============================================================================
class SurvivalCoopLogicHandler expands GGameLogicHandler
	config(SurvivalCoopMut);

// Class-wide constants
const LOG_WATERMARK = 'SurvivalCoop'; 
const TIME_UNTIL_LOCKOUT = 180;	// 2 minute for new players to join
const TAG_SERVEREVENT = 'ServerEvent';
Var GenericGame Game;

// Class-wide member variables
// gameplay management
var survivalcoopplayer localPlayer;
var bool bNewRound;					// If true, a new round has already begin, and new players are denied the possibility of playing this round.
var bool bWipedOut;				// If true, everyone was wiped out!
var() config sound EndGameSound; //the sound that plays when the level ends
var() bool AmmoRewards; //does the server reward you ammo when you win the round?
var() float AmmoMult;
var pawn Temp;		
var SurvivalGRI sGRI;
var bool byouFuckedUp;

// client management
var int numPasses;			// Running count of how many new players we have seen thus far
var string passwordBank[32];	// Number of new clients we can support

// Inherited from CoopGame
var() bool bNoFriendlyFire;
var bool bSpecialFallDamage;

// idle kicker
var() config int KickTime; // in sec
var bool Initialized;
var int LastCheck;
var rotator CurRotation;

struct FireRecord   //player fire records
{
	var bool bFireLog; // is player monitored - heehee
	var bool bAboutToBeKicked;
	var bool bJustDied; // Check if the player just died
	var int LastFireTime; // start of Fire monitor
	var int LastRotateTime; // start of Rotation monitor
	var rotator TestRotation; // Test if Rotation changed within 5 seconds
	var Weapon OldWeapon; // What weapon the player is using
};

var FireRecord FireRecords[50]; //keep record of max 50 players

//============ Initialization ================//
function bool IsRelevant( actor Other )
{
    return true;
}

function Initialize(GenericGame GGame)
{
	Super.Initialize(GGame);
	Level.Game.StartTime = 0;
	bNoFriendlyFire = true;
	SetGameSpeed(Level.Game.GameSpeed);
	Level.bNoCheating = Level.Game.bNoCheating;
}

simulated function postBeginPlay()
{
  super.postBeginPlay();

  numPasses = 0;
  bNewRound = false;
  bWipedOut = false;
  if (!Initialized)
  {
      Initialized = true;
	  Enable('Tick');
  }

  if( role==role_authority )
    settimer( TIME_UNTIL_LOCKOUT, false );
    
  if ( Level.Title ~= "The Source Antechamber" )
  {
     bSpecialFallDamage = true;
	log("reduce fall damage");
  }
}

// ProcessServerTravel:
//	Optional handling of ServerTravel for network games.
function ProcessServerTravel( string URL, bool bItems )
{
	local Playerpawn P;
	// We call PreClientTravel directly on any local PlayerPawns (ie listen server)
	log("ProcessServerTravel:"@URL);
	foreach AllActors( class'PlayerPawn', P )
		if( NetConnection(P.Player)!=None )
			P.ClientTravel( URL, TRAVEL_Relative, bItems );
		else
			P.PreClientTravel();

	// Switch immediately if not networking.
	if( Level.NetMode!=NM_DedicatedServer && Level.NetMode!=NM_ListenServer )
		Level.NextSwitchCountdown = 0.0;
}

function serverTravel( string URL )
{
  switchlevel(none,url);
}

function switchlevel( playerpawn player, string URL )
{	  
	Log("Performing switchlevel to URL: "$URL,LOG_WATERMARK);
	gotostate('PreGameEnd','End');
}

// RestartGame:
//	Restart the game.
function RestartGame()
{
	ProcessServerTravel(GetURLMap(),True);
}

// LevelEnd:
//	Called when the level is about to end.
function LevelEnd(GPlayer Other, coerce string URL)
{
    local survivalcoopplayer tempplayer;
    local survivalcoopplayer P;
    local int i;
    
     i = 0;
	// hack to skip end game in coop play
	if ( left(URL,6) ~= "endgame")
	{
		Level.ServerTravel( "Vortex2", false);
		return;
	}
	
	if(bYouFuckedUP)
	{
	  URL=self.getUrlMap();
      level.game.broadcastMessage("Game over!",true,'FlashingCriticalEvent');	
      sGRI.consecutiveWins = 0;	
      sGRI.SaveConfig();
     }
     
     else
     {
       forEach allActors(class'SurvivalCoopPlayer', tempplayer)
       {
          tempplayer.bMovable = False;
          tempplayer.ViewTarget = Other;
          tempplayer.clientplaysound(EndGameSound);
          tempplayer.bholster = false;
          Level.Game.broadcastmessage(Other.PlayerReplicationInfo.PlayerName$" has ended the level!");
          i++;
       }
       sGRI.consecutivewins = sGRI.consecutivewins + 1;
       sGRI.SaveConfig();
    
           if(i < 3)
            {
                level.game.difficulty=1;
                log("The server is now on Medium difficulty");
            }
            else if (i >= 3 && i <= 4)
            {
                level.game.difficulty=2;
                log("The server is now on Hard difficulty");
            }
            else if (i >= 5 && i <= 7)
            {
                level.game.difficulty=3;
                log("The server is now on Unreal difficulty");
            }
            else if (i >= 8 && i <= 12)
            {
                level.game.difficulty=4;
                log("The server is now on lvl 4 difficulty");
            }
            else
            {
                level.game.difficulty=5;
                log("The server is now on lvl 5 difficulty");
            }
   }
   	 Level.ServerTravel( URL, true );
}


/* FIXME
//============ Client Management ================//
// Override to put new players in the correct state
function MutateClientLoadedPassword( XPRI ClientXPRI )
{
  local xplayer newplayer;
  local bool bReturningPlayer;
  local Ammo A;
  local int AddAmmo;
  local actor target;

  newplayer = xplayer(clientXPRI.owner);
  
  if( newplayer == none )
    return; // Why the hell would this ever happen? Check just in case.
  
  bReturningPlayer = isInBank( ClientXPRI.newxCoopPassword );		// Is this a returning player?
  
    if (bReturningPlayer || bNewRound)
    {
      if( bNewRound ) 
      {
	    // New player. Fuck off.
        newplayer.Died(newplayer, 'Decaptiated', Newplayer.Location);
        newplayer.bMovable = false;
        newplayer.drawtype = DT_None;	// They're done, screw it.
        //bNewRound = true;
        viewRandom(newplayer);
        if( bReturningPlayer )
    	    xplayer(ClientXPRI.Owner).clientMessage("Sorry, you cannot reconnect to play again this round.",TAG_SERVEREVENT,true);
  	    xplayer(clientxpri.owner).clientmessage("Please wait until this round has completed.",TAG_SERVEREVENT,true);
      }
    }
    else
    {
  	// Not a new round - let the player play.
  	registerNewPlayer( ClientXPRI.newxCoopPassword );
    }
    
    xplayer(ClientXPRI.Owner).bholster = false;  //added to prevent error from InventoryData travel in 227g_28 revision
        
    if (xplayer(ClientXPRI.Owner).health <= 100)
    {
        xplayer(ClientXPRI.Owner).health = 100;
    }
    else if (xplayer(ClientXPRI.Owner).health > 100 && xplayer(ClientXPRI.Owner).health <= 150 )
    {
        xplayer(ClientXPRI.Owner).health = 150;
    }
    else xplayer(ClientXPRI.Owner).health = 200;
    
    ForEach AllActors(class'Ammo',A)
    {
        if (A.Owner == NewPlayer && AmmoRewards == true) 
        {
            AddAmmo = (A.MaxAmmo * AmmoMult);

            if(A.AmmoAmount + AddAmmo < A.MaxAmmo)
            {
                 A.AmmoAmount = (A.AmmoAmount + AddAmmo);
            }
            else A.AmmoAmount = A.MaxAmmo;
        }

    }

}

// Adds this player's pass to the internal array
function registerNewPlayer( String newPass )
{
  log("Registering new client's password: "$newPass,LOG_WATERMARK);
  passwordBank[numPasses] = newPass;	// Register it
  numPasses++;
}

// Adds this player's pass to the internal array
function registerNewPlayer( String newPass )
{
  log("Registering new client's password: "$newPass,LOG_WATERMARK);
  passwordBank[numPasses] = newPass;	// Register it
  numPasses++;
}

// Checks if this pass is already in the bank
function bool isInBank( String testPass )
{
  local int index;	// Used in iteration
  
  for( index = 0; index < numPasses; index++ )
  {
    if( passwordBank[index] == testPass ) 
      return true;		// We found it
  }
  
  return false;	// Mission Failed.
}
*/

//============= Gameplay Logic ================//
// Override to detect player deaths

function Killed(pawn Killer, pawn Killed, name DamageType)
{
    local int numPlayers;
    local survivalcoopplayer P, RevivePlayer;
    local actor target;
    local SurvivalPRI PRI;
    local int Score;
	local bool bFirstCheck;

	
	// Should we make them immobile?
    if (Killed.isA('survivalcoopplayer'))
    {
    	survivalPRI( survivalcoopplayer(Killed).playerReplicationInfo ).bIsDead = true;	// Flag them as dead.
		survivalcoopPlayer(Killed).bHolster = true;
        SurvivalCoopPlayer(Killed).bMovable = false;
        Killed.drawtype = DT_None;	// They're done, screw it.
        //bNewRound = true;
        viewRandom(survivalcoopplayer(Killed));
    }
    else return; // no need to check
    
    numPlayers = 0;
    foreach AllActors(class'survivalcoopplayer', P)
    {
        if (P.bMovable)
            numPlayers++;
        else if( p.viewTarget == Killed )
            p.viewTarget = target;		// Fix this player's view

    }
    
    if(numplayers==1)
    {
	  bFirstCheck = true;
	  
      foreach AllActors(class'survivalcoopplayer', P)
      {
	    if(P.isinstate('Dying'))
		{
		  if(bFirstCheck)
		  {
		    Score = survivalPRI(P.playerReplicationInfo).Score; 
			bFirstCheck=false;
		  }
		  
          if(Score >= survivalPRI(P.playerReplicationInfo).Score && 
             survivalPRI(P.playerReplicationInfo).Score != 0)
          {
            RevivePlayer = P;
          }
		  
		  Score = survivalPRI(P.playerReplicationInfo).Score; 
	    }
      }
	  
	  // FIXME Class'SCoopCommands'.Static.Revive(RevivePlayer,RevivePlayer.PlayerReplicationInfo.ID);
    }

    if(numPlayers==0)
    {
      byouFuckedUP=true;
      RequestLevelEnd(self.GetURLMap(), false);	// fuck it
    }

}

function ScoreKill(pawn Killer, pawn Other)
{
	Other.DieCount++;
	if( (killer == Other) || (killer == None) )
		Other.PlayerReplicationInfo.Score -= 1;
	else if ( killer != None )
	{
		killer.killCount++;
		if ( killer.PlayerReplicationInfo != None )
			killer.PlayerReplicationInfo.Score += 1;
	}
}	

// PickupQuery:
//	Called when pawn has a chance to pick Item up (i.e. when 
// the pawn touches a weapon pickup). Should return true if 
// he wants to pick it up, false if he does not want it.
function bool PickupQuery( Pawn Other, Inventory item )
{
	if ( Other.Inventory == None )
		return true;
	else
		return !Other.Inventory.HandlePickupQuery(Item);
}

// ShouldRespawn:
//	Return whether an item should respawn.
function bool ShouldRespawn( actor Other )
{
	if ( Other.IsA('Weapon') && !Weapon(Other).bHeldItem && (Weapon(Other).ReSpawnTime != 0) )
	{
		Inventory(Other).ReSpawnTime = 1.0;
		return true;
	}
	return false;
}


// Called by gametype when game is REALLY over.
function levelEnded()
{
  local pawn P;
  local SurvivalGRI sGRI;
  
  foreach allactors(class'SurvivalGRI', sGRI)
  {
	if( bWipedOut == false )
	{
	  sGRI.ConsecutiveWins++;
      sGRI.saveConfig();
	}
	else 
    {   

        sGRI.saveConfig();
    }

	gameAndLevelEnd();
  }
  
}

// gets called by both game and level ends.
function gameAndLevelEnd() 
{
    local survivalcoopplayer P;
    local int i;
    i = 0;

    foreach allActors(class'survivalcoopplayer',P)
    {
        if ( bWipedOut == true )
        {
            P.clientplaysound(EndGameSound);
            p.bholster = false;
        }

        i++;
    }
    
           if(i < 3)
            {
                consolecommand("set xcoop gamedifficulty 2"); //hard difficulty
                log("The server is now on Hard difficulty");
            }
            else if (i >= 3 && i <= 4)
            {
                consolecommand("set xcoop gamedifficulty 3");
                log("The server is now on Unreal difficulty");
            }
            else if (i >= 5 && i <= 7)
            {
                consolecommand("set xcoop gamedifficulty 4");
                log("The server is now on lvl 4 difficulty");
            }
            else if (i >= 8 && i <= 12)
            {
                consolecommand("set xcoop gamedifficulty 5");
                log("The server is now on lvl 5 difficulty");
            }
            else
            {
                consolecommand("set xcoop gamedifficulty 6");
                log("The server is now on lvl 6 difficulty");
            }
}


// Assigns this player a random player to view from.
function viewRandom(PlayerPawn TargetReciever)
{
  local survivalcoopplayer P; 
  local survivalcoopplayer Players[32];
  local int i,x;
  foreach allActors(class'survivalcoopplayer',P)
    if(p.health > 0)
    {
     Players[x++] = P;
    }
    i = rand(x);
	if (TargetReciever != None)
		TargetReciever.ViewTarget = Players[i];
}

// Locks the server from new clients.. new clients can't play this round (they can still spectate, however)
function timer()
{
    local int i;
    local survivalcoopplayer P;
 
    ForEach AllActors(class'survivalcoopplayer', P)
    {
        i++;
    }
    if(i > 1)
    { 
	    bNewRound = true;
    }
    else if( role==role_authority )
    settimer( TIME_UNTIL_LOCKOUT, false );
}

//============================
// Player Management Handling

// LoadReplicationInfos:
//	Called when the main gametype wishes to have its info fields filled. Return a
//	GameReplicationInfo object through the first out parameter, and the class to be
//	used for PlayerReplicationInfos in the second out parameter.

Function bool PreLogin(String Options, Out String Error)
{
	return true;
}

// Login:
//	Called when a player requests login. 
Function GPlayer Login(String Portal, String Options, class<PlayerPawn> SpawnClass, out string Error )
{
	local NavigationPoint StartSpot;
	local PlayerPawn      NewPlayer, TestPlayer;
	local Pawn            PawnLink;
	local string          InName, InPassword, InSkin, InFace;
	local byte            InTeam;
	local string		  InClassStr;

	// Make sure there is capacity. (This might have changed since the PreLogin call).
	if( (Level.NetMode != NM_Standalone) && (Level.Game.MaxPlayers>0) && (Level.Game.NumPlayers>=Level.Game.MaxPlayers) )
	{
		return None;
	}

	// Get URL options.
	InName     = Left(Level.Game.ParseOption ( Options, "Name"), 20);
	InTeam     = Level.Game.GetIntOption( Options, "Team", 255 ); // default to "no team"
	InPassword = Level.Game.ParseOption ( Options, "Password" );
	InSkin	   = Level.Game.ParseOption ( Options, "Skin"    );
	InFace     = Level.Game.ParseOption ( Options, "Face"    );
	log( "Login:" @ InName );
	if( InPassword != "" )
		log( "Password"@InPassword );
	 
	// Find a start spot.
	StartSpot = FindPlayerStart( InTeam, Portal );

	if( StartSpot == None )
	{
		return None;
	}

	// Try to match up to existing unoccupied player in level,
	// for savegames and coop level switching.
	for( PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.NextPawn )
	{
		TestPlayer = PlayerPawn(PawnLink);
		if
		(	TestPlayer!=None
		&&	TestPlayer.Player==None )
		{
			if
			(	(Level.NetMode==NM_Standalone)
			||	(TestPlayer.PlayerReplicationInfo.PlayerName~=InName && TestPlayer.Password~=InPassword) )
			{
				// Found matching unoccupied player, so use this one.
				NewPlayer = TestPlayer;
				break;
			}
		}
	}
	
	NewPlayer = SurvivalCoopPlayer(NewPlayer);

	// In not found, spawn a new player.
	if( NewPlayer==None )
	{
		InClassStr = caps(string(SpawnClass));
		if( InStr(InClassStr,"FEMALEONE") != -1 )
		{
			SpawnClass = class'SurvivalCoopFemaleOne';
		}
		else if( InStr(InClassStr,"FEMALETWO") != -1 )
		{
			SpawnClass = class'SurvivalCoopFemaleTwo';
		}
		else if( InStr(InClassStr,"MALEONE") != -1 )
		{
			SpawnClass = class'SurvivalCoopMaleOne';
		}
		else if( InStr(InClassStr,"MALETWO") != -1 )
		{
			SpawnClass = class'SurvivalCoopMaleTwo';
		}
		else if( InStr(InClassStr,"MALETHREE") != -1 )
		{
			SpawnClass = class'SurvivalCoopMaleThree';		
		}
		else if( InStr(InClassStr,"NALI") != -1 )
		{
			SpawnClass = class'SurvivalCoopMaleTwo';
		}
		else if( InStr(InClassStr,"SKAARJ") != -1 )
		{
			SpawnClass = class'SurvivalCoopMaleTwo';
		}
	
		// Make sure this kind of player is allowed.
		if ( ((Level.Game.bHumansOnly || Level.bHumansOnly) && !SpawnClass.Default.bIsHuman) || 
			!ClassIsChildOf(SpawnClass,class'SurvivalCoopPlayer') )
			SpawnClass = Level.Game.DefaultPlayerClass;

		NewPlayer = Spawn(SpawnClass,,,StartSpot.Location,StartSpot.Rotation);
		if( NewPlayer!=None )
			NewPlayer.ViewRotation = StartSpot.Rotation;
				if ( NewPlayer != None )
	}

	// Handle spawn failure.
	if( NewPlayer == None )
	{
		log("Couldn't spawn player at "$StartSpot);
		return None;
	}

	if ( InSkin != "" )
		NewPlayer.static.SetMultiSkin(NewPlayer, InSkin, InFace, InTeam);

	if ( !ChangeTeam(newPlayer, InTeam) )
	{
		return None;
	}

	// Init player's information.
	NewPlayer.ClientSetRotation(NewPlayer.Rotation);
	if( InName=="" )
		InName=Level.Game.DefaultPlayerName;
	if( Level.NetMode!=NM_Standalone || NewPlayer.PlayerReplicationInfo.PlayerName==Level.Game.DefaultPlayerName )
		ChangeName( NewPlayer, InName, false );

	// Init player's replication info
	NewPlayer.GameReplicationInfo = Level.Game.GameReplicationInfo;

	// If we are a server, broadcast a welcome message.
	if( Level.NetMode==NM_DedicatedServer || Level.NetMode==NM_ListenServer )
		Level.Game.BroadcastMessage( NewPlayer.PlayerReplicationInfo.PlayerName$Level.Game.EnteredMessage, false );

	// Teleport-in effect.
	StartSpot.PlayTeleportEffect( NewPlayer, true );

	// Set the player's ID.
	NewPlayer.PlayerReplicationInfo.PlayerID = Level.Game.CurrentID++;

	if ( !NewPlayer.IsA('Spectator') )
		Level.Game.NumPlayers++;
	return SurvivalCoopPlayer(newPlayer);
}

function NavigationPoint FindPlayerStart( byte Team, optional string incomingName )
{
	local PlayerStart Dest, Candidate[8], Best;
	local float Score[8], BestScore, NextDist;
	local pawn OtherPlayer;
	local int i, num;
	local Teleporter Tel;

	num = 0;
	//choose candidates	
	foreach AllActors( class 'PlayerStart', Dest )
	{
		if ( (Dest.bSinglePlayerStart || Dest.bCoopStart) && !Dest.Region.Zone.bWaterZone )
		{
			if (num<4)
				Candidate[num] = Dest;
			else if (Rand(num) < 4)
				Candidate[Rand(4)] = Dest;
			num++;
		}
	}
	
	if (num>4) num = 4;
	else if (num == 0)
		return None;
		
	//assess candidates
	for (i=0;i<num;i++)
		Score[i] = 4000 * FRand(); //randomize
		
	foreach AllActors( class 'Pawn', OtherPlayer )
	{
		if (OtherPlayer.bIsPlayer)
		{
			for (i=0;i<num;i++)
			{
				NextDist = VSize(OtherPlayer.Location - Candidate[i].Location);
				Score[i] += NextDist;
				if (NextDist < OtherPlayer.CollisionRadius + OtherPlayer.CollisionHeight)
					Score[i] -= 1000000.0;
			}
		}
	}
	
	BestScore = Score[0];
	Best = Candidate[0];
	for (i=1;i<num;i++)
	{
		if (Score[i] > BestScore)
		{
			BestScore = Score[i];
			Best = Candidate[i];
		}
	}			
				
	return Best;
}

// AddDefaultInventory:
//	Spawn any default inventory for the player.
function AddDefaultInventory( pawn PlayerPawn )
{
	local Translator newTranslator;
	local weapon newWeapon, newWeapon2;

	if ( Level.DefaultGameType != class'VRikersGame' ) 
	{
		PlayerPawn.JumpZ = PlayerPawn.Default.JumpZ * PlayerJumpZScaling();
		 
		if( PlayerPawn.IsA('Spectator') )
			goto Next;
	
		// Spawn default weapon.
		if( Level.Game.DefaultWeapon==None || PlayerPawn.FindInventoryType(Level.Game.DefaultWeapon)!=None )
			goto Next;
	
		newWeapon = Spawn(class'DispersionPistol');
		newWeapon2 = spawn(class'automag');
		
		if( newWeapon != None && newWeapon2 != None)
		{
	      newWeapon.Instigator = PlayerPawn;
		  newWeapon.BecomeItem();
		  PlayerPawn.AddInventory(newWeapon);
		  newWeapon.WeaponSet(PlayerPawn);
		  newWeapon.GiveAmmo(PlayerPawn);
		  newWeapon.bringup();
		  newWeapon.SetSwitchPriority(PlayerPawn);
		  

		newWeapon2.Instigator = PlayerPawn;
		newWeapon2.BecomeItem();
		PlayerPawn.AddInventory(newWeapon2);
		newWeapon2.GiveAmmo(PlayerPawn);
		newWeapon2.GiveAmmo(PlayerPawn);
		newWeapon2.GiveAmmo(PlayerPawn);
		newWeapon2.GiveAmmo(PlayerPawn);
		newWeapon2.GiveAmmo(PlayerPawn);
		

		}
		
	}
	
Next:
	// Spawn translator.
	if( PlayerPawn.IsA('Spectator') || PlayerPawn.FindInventoryType(class'Translator') != None )
		return;
	newTranslator = Spawn(class'Translator',,, Location);
	if( newTranslator != None )
	{
		newTranslator.bHeldItem = true;
		newTranslator.GiveTo( PlayerPawn );
		PlayerPawn.SelectedItem = newTranslator;
		newTranslator.PickupFunction(PlayerPawn);
	}
}

// ChangeName:
//	Called after a pawn has requested to change their name.
function ChangeName( Pawn Other, coerce string S, bool bNameChange )
{
	if( S == "" )
		return;
	Other.PlayerReplicationInfo.PlayerName = S;
	if( bNameChange )
		Other.ClientMessage( Level.Game.NameChangedMessage $ Other.PlayerReplicationInfo.PlayerName );
}

// ChangeTeam:
//	Return whether a team change is allowed.
function bool ChangeTeam(Pawn Other, int N)
{
	Other.PlayerReplicationInfo.Team = N;
	return true;
}

// StartPlayer:
//	Start a player.
function StartPlayer(PlayerPawn Other)
{
	if( Level.NetMode==NM_DedicatedServer || Level.NetMode==NM_ListenServer || !Level.Game.bRestartLevel )
		Other.GotoState('PlayerWalking');
	else
		Other.ClientTravel( "?restart", TRAVEL_Relative, false );
}

// RestartPlayer:
//	Called when a player should be restarted.
function bool RestartPlayer( pawn aPlayer )
{
	local NavigationPoint startSpot;
	local bool foundStart;

	if( Level.Game.bRestartLevel && Level.NetMode!=NM_DedicatedServer && Level.NetMode!=NM_ListenServer )
		return true;

	startSpot = FindPlayerStart(aPlayer.PlayerReplicationInfo.Team);
	if( startSpot == None )
		return false;
		
	foundStart = aPlayer.SetLocation(startSpot.Location);
	if( foundStart )
	{
		startSpot.PlayTeleportEffect(aPlayer, true);
		aPlayer.SetRotation(startSpot.Rotation);
		aPlayer.ViewRotation = aPlayer.Rotation;
		aPlayer.Acceleration = vect(0,0,0);
		aPlayer.Velocity = vect(0,0,0);
		aPlayer.Health = aPlayer.Default.Health;
		aPlayer.SetCollision( true, true, false );
		aPlayer.ClientSetLocation( startSpot.Location, startSpot.Rotation );
		aPlayer.bHidden = false;
		aPlayer.DamageScaling = aPlayer.Default.DamageScaling;
		aPlayer.SoundDampening = aPlayer.Default.SoundDampening;
		AddDefaultInventory(aPlayer);
	}

	return foundStart;
}

// Logout:
//	Called when a player is leaving the game.
function Logout( pawn Exiting )
{
	if ( Exiting.IsA('PlayerPawn') && !Exiting.IsA('Spectator') )
	{
		Level.Game.NumPlayers--;
	}

	if( Level.NetMode==NM_DedicatedServer || Level.NetMode==NM_ListenServer )
		BroadcastMessage( Exiting.PlayerReplicationInfo.PlayerName$Level.Game.LeftMessage, false );
}

// HandleCustomCommand:
//	Called when a player wishes to execute a custom command.
Function HandleCustomCommand(GPlayer Other, name Command, optional coerce string Params)
{
local class<actor> NewClass;
local Pawn P;
local actor vother, first;
local bool bFound;
local string Next;

	if( Command == 'Say' || Command == 'TeamSay' )
	{
		for( P=Level.PawnList; P!=None; P=P.nextPawn )
			if( P.bIsPlayer )
				P.TeamMessage( Other.PlayerReplicationInfo, Params, 'Say' );
	}
	else if( Command == 'Summon' )
	{
		log( "Fabricate " $ Params );
		if( instr(Params,".")==-1 )
		Params = "UnrealI." $ Params;
		NewClass = class<actor>( DynamicLoadObject( Params, class'Class' ) );
		if( NewClass!=None )
			Spawn( NewClass,,,Other.Location + 72 * Vector(Other.ViewRotation) + vect(0,0,1) * 15 );
	}
	else if( Command == 'ViewClass')
	{
		Next = Params;
		Next = Left(Next,InStr(Next," "));
		first = None;
		NewClass = class<actor>( DynamicLoadObject(Next,class'class') );
		ForEach AllActors( NewClass, vother )
		{
			if ( (first == None) && (vother != other) )
			{
				first = vother;
				bFound = true;
			}
			if ( vother == Other.ViewTarget ) 
				first = None;
		}  

		if ( first != None )
		{
			if ( first.IsA('Pawn') && Pawn(first).bIsPlayer && (Pawn(first).PlayerReplicationInfo.PlayerName != "") )
				Other.ClientMessage(Other.ViewingFrom$Pawn(first).PlayerReplicationInfo.PlayerName, 'Event', true);
			else
				Other.ClientMessage(Other.ViewingFrom$first, 'Event', true);

			Other.ViewTarget = first;
		}
		else
		{
			if ( bFound )
				Other.ClientMessage(Other.ViewingFrom$Other.OwnCamera, 'Event', true);
			else
				Other.ClientMessage(Other.FailedView, 'Event', true);
			Other.ViewTarget = None;
		}

		if ( ( Other.ViewTarget != None ) )
			Other.ViewTarget.BecomeViewTarget();
	}
}

// AllowsBroadcast:
//	Called when a message wishes to be broadcasted. Return false to reject.
function bool AllowsBroadcast( Actor Broadcaster, int Len )
{
  return true;
}

// HandleMessage:
//	Called when a message has been sent.
Function HandleMessage( GPlayer Sender, Name MsgType, String Msg )
{
	// Unused
}

// AllowLevelSwitch:
//	Return true to allow a level to switch. False to reject. Called before mutators or gamelogic can act on request.
function bool AllowLevelSwitch( GPlayer Other, String URL, bool bTeleporterEnd )
{
	return true;
}

// AllowSpectate:
//	Return whether Viewer is allowed to spectate from the
// point of view of ViewTarget.
function bool AllowSpectate( pawn Viewer, actor ViewTarget )
{
	return true;
}

// SetGameSpeed:
//	Called when an actor requests gamespeed to be changed.
function SetGameSpeed( Float T )
{
	Level.Game.GameSpeed = FMax(T, 0.1);
	Level.TimeDilation = Level.Game.GameSpeed;
}

// PlayerJumpZScaling:
//	Return the player jumpZ scaling for this gametype
function float PlayerJumpZScaling()
{
	return 1.0;
}

// DiscardInventory:
//	Discard a player's inventory after he dies.
function DiscardInventory( Pawn Other )
{
	local actor dropped;
	local inventory Inv;
	local weapon weap;
	local float speed;

	if( Other.DropWhenKilled != None )
	{
		dropped = Spawn(Other.DropWhenKilled,,,Other.Location);
		Inv = Inventory(dropped);
		if ( Inv != None )
		{ 
			Inv.RespawnTime = 0.0; //don't respawn
			Inv.BecomePickup();		
		}
		if ( dropped != None )
		{
			dropped.RemoteRole = ROLE_DumbProxy;
			dropped.SetPhysics(PHYS_Falling);
			dropped.bCollideWorld = true;
			dropped.Velocity = Other.Velocity + VRand() * 280;
		}
		if ( Inv != None )
			Inv.GotoState('PickUp', 'Dropped');
	}					
	if( (Other.Weapon!=None) && (Other.Weapon.Class!=Level.Game.DefaultWeapon) 
		&& Other.Weapon.bCanThrow )
	{
		speed = VSize(Other.Velocity);
		weap = Other.Weapon;
		weap.Velocity = Normal(Other.Velocity/speed + 0.5 * VRand()) * (speed + 280);
		Other.TossWeapon();
		if ( weap.PickupAmmoCount == 0 )
			weap.PickupAmmoCount = 1;
	}
	Other.Weapon = None;
	Other.SelectedItem = None;	
	for( Inv=Other.Inventory; Inv!=None; Inv=Inv.Inventory )
		Inv.Destroy();
}


// PlaySpawnEffect:
//	Play an inventory respawn effect.
function float PlaySpawnEffect( inventory Inv )
{
	Playsound(sound'RespawnSound');
	if ( !Level.Game.bCoopWeaponMode || !Inv.IsA('Weapon') )
	{
		spawn( class 'ReSpawn',,, Inv.Location );
		return 0.3;
	}
	return 0.0;
}

// PlayTeleportEffect:
//	Play a teleporting special effect.
function PlayTeleportEffect( actor Incoming, bool bOut, bool bSound)
{
}

Function InitGameReplicationInfo(GameReplicationInfo GameReplicationInfo)
{
    sGRI = SurvivalGRI(GameReplicationInfo);
	GameReplicationInfo.bTeamGame = true;
	GameReplicationInfo.GameName = "Coop";
}

//used by idle kicker to keep track of time
function Tick(float DeltaTime)
{
	local int CurrentTime;

	CurrentTime = Level.TimeSeconds;
	if (CurrentTime>LastCheck)
	{
		CheckForFire();
		LastCheck = CurrentTime;
	}
	SpecialCheck();
}

function SpecialCheck()
{	
	local Pawn P;
	local survivalcoopplayer PP;
	local vector TestBob;
	local int PID;
	local int CurTime;
	TestBob = vect(0,0,0);

	for(P=level.pawnlist; P!=none ; P=P.nextpawn)
	{
		if ( P.bIsPlayer && P.IsA('survivalcoopplayer') && !P.PlayerReplicationInfo.bIsABot && !P.PlayerReplicationInfo.bIsSpectator && !Level.Game.bGameEnded )
		{
			PID = P.PlayerReplicationInfo.PlayerID;
			PP=survivalcoopplayer(P);
			if (P.ViewRotation.Pitch < 32768)
				CurRotation.Pitch = 65536 + P.ViewRotation.Pitch;
			else
				CurRotation.Pitch = P.ViewRotation.Pitch;
			CurTime = Level.TimeSeconds;
			if (P.Health < 1)
			{
				FireRecords[PID].bJustDied=true;
			}
			else if (FireRecords[PID].bJustDied && P.Health > 0)
			{
				FireRecords[PID].OldWeapon = P.Weapon;
				FireRecords[PID].bJustDied=false;
				FireRecords[PID].LastRotateTime = CurTime;
				if (P.ViewRotation.Pitch < 32768)
					FireRecords[PID].TestRotation.Pitch = 65536 + P.ViewRotation.Pitch;
				else
					FireRecords[PID].TestRotation.Pitch = P.ViewRotation.Pitch;
			}
			if(FireRecords[PID].bAboutToBeKicked)
				if((P.bFire!=0 || P.bAltFire!=0 || P.bDuck!=0 || PP.bPressedJump!=False || P.Acceleration!=TestBob || P.Weapon!=FireRecords[PID].OldWeapon || ((CurRotation.Pitch) > (FireRecords[PID].TestRotation.Pitch+150) && (CurRotation.Pitch) < (FireRecords[PID].TestRotation.Pitch-150))) || (P.PlayerReplicationInfo.HasFlag!=None))
				{
					if (P.Health > 0 || PP.bPressedJump!=False || P.bFire!=0)
					{
					FireRecords[PID].bFireLog = false;
					FireRecords[PID].bAboutToBeKicked = false;
					}
				}
			FireRecords[PID].OldWeapon = P.Weapon;
			CurTime = Level.TimeSeconds;
			if ((CurTime-FireRecords[PID].LastRotateTime) > 5)
			{
				FireRecords[PID].LastRotateTime = CurTime; // record time every 5 seconds
				if (P.ViewRotation.Pitch < 32768)
					FireRecords[PID].TestRotation.Pitch = 65536 + P.ViewRotation.Pitch;
				else
					FireRecords[PID].TestRotation.Pitch = P.ViewRotation.Pitch;
			}
		}
	}
}

function CheckForFire()
{	
	local Pawn P;
	local survivalcoopplayer PP;
	local vector TestBob;
	local int PID;
	local int CurTime;
	TestBob = vect(0,0,0);

	for(P=level.pawnlist; P!=none ; P=P.nextpawn)
	{	
		if ( P.bIsPlayer && P.IsA('survivalcoopplayer') && !P.PlayerReplicationInfo.bIsABot && !P.PlayerReplicationInfo.bIsSpectator && !Level.Game.bGameEnded && P.bMovable )
		{
			PP=survivalcoopplayer(P);
			PID = P.PlayerReplicationInfo.PlayerID; // get player ID
			if (P.ViewRotation.Pitch < 32768)
				CurRotation.Pitch = 65536 + P.ViewRotation.Pitch;
			else
				CurRotation.Pitch = P.ViewRotation.Pitch;
			CurTime = Level.TimeSeconds;
			if (P.Health < 1)
			{
				FireRecords[PID].bJustDied=true;
			}
			else if (FireRecords[PID].bJustDied && P.Health > 0)
			{
				FireRecords[PID].OldWeapon = P.Weapon;
				FireRecords[PID].LastRotateTime = CurTime;
				if (P.ViewRotation.Pitch < 32768)
					FireRecords[PID].TestRotation.Pitch = 65536 + P.ViewRotation.Pitch;
				else
					FireRecords[PID].TestRotation.Pitch = P.ViewRotation.Pitch;
			}
			if((((P.bFire==0) && (P.bAltFire==0) && (P.bDuck==0) && (PP.bPressedJump==False) && (P.Acceleration==TestBob) && (P.Weapon==FireRecords[PID].OldWeapon) && (P.PlayerReplicationInfo.HasFlag==None) && (CurRotation.Pitch) < (FireRecords[PID].TestRotation.Pitch+200) && (CurRotation.Pitch) > (FireRecords[PID].TestRotation.Pitch-200))) || (P.Health < 1 && (P.bFire==0) && (PP.bPressedJump==False)))
			{
				CurTime = Level.TimeSeconds;
				if(FireRecords[PID].bFireLog)
				{
					if ((CurTime-FireRecords[PID].LastFireTime) > KickTime)
					{
						P.ConsoleCommand("Suicide");
						FireRecords[PID].bFireLog = false;
					}
					else if	((CurTime-FireRecords[PID].LastFireTime) > (KickTime-10))
					{
						FireRecords[PID].bAboutToBeKicked = true;
						if (CurTime%2==0)
						{
							//P.ReceiveLocalizedMessage( class'KickIdlePlayersMsg2',1 );
							P.ClientMessage(">> " $ P.PlayerReplicationInfo.PlayerName $ " You will be KILLED for not playing!!");
						}
					}
				}
				else
				{
					FireRecords[PID].bFireLog = true; // monitor player
					FireRecords[PID].LastFireTime = CurTime; // start time
				}
			}
			else if (FireRecords[PID].bFireLog)
			{
				FireRecords[PID].bFireLog = false;
			}
		}
	}
}

function FinalizeLogin( GPlayer NewPlayer )
{
   NewPlayer.setcollision(true,true,false);
}

defaultproperties
{
				EndGameSound=Sound'UnrealShare.Generic.Teleport1'
				KickTime=60
				GameName="Coop Game"
				bHumansOnly=True
				bCoopWeaponMode=True
				DefaultPlayerClass=Class'AST.SurvivalCoopPlayer'
				ScoreBoardType=Class'AST.SurvivalScoreBoard'
				HUDType=Class'AST.SurvivalCoopHUD'
				GameReplicationInfoClass=Class'AST.SurvivalGRI'
}
